/*
Copyright (C) Chao-Kuo LIn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#define STRICT
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include "ckMouseHook.h"
#include <cassert>

#pragma data_seg(".ckMouseHookShareData")
HWND g_HotspotHWND = NULL;
HHOOK g_MouseHook = NULL;
DWORD g_Message = 0;
#pragma data_seg()

HINSTANCE g_hInstance = NULL;


LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam);


BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(lpvReserved);
	if(fdwReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hinstDLL;
		DisableThreadLibraryCalls(g_hInstance);
	}
	else if(fdwReason == DLL_PROCESS_DETACH)
	{
		//unload our stuff
		SetMouseHook(FALSE);
	}

	return TRUE;
}

//initialize w/ your hwnd and the message you want to recieve
void InitializeHook(HWND hwnd, DWORD message)
{
	g_HotspotHWND = hwnd;
	g_Message = message;
}

//sethook == TRUE if you want to set a hook
BOOL SetMouseHook(BOOL sethook)
{
	if(g_HotspotHWND == NULL || g_Message == 0 || g_hInstance == NULL)
		return FALSE;

	if(sethook)
	{
		if(g_MouseHook == NULL)
			g_MouseHook = SetWindowsHookEx(WH_MOUSE , MouseProc, g_hInstance, 0);
		return g_MouseHook != NULL;
	}
	else
	{
		if(UnhookWindowsHookEx(g_MouseHook))
			g_MouseHook = NULL;
		
		return g_MouseHook == NULL;
	}


}

LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	MOUSEHOOKSTRUCT *ptr = (MOUSEHOOKSTRUCT *)lParam;

	if(nCode >= 0 && ptr != NULL && g_HotspotHWND != NULL)
	{
		SendMessage(g_HotspotHWND, g_Message, ptr->pt.y, ptr->pt.x);
	}

	return CallNextHookEx(g_MouseHook, nCode, wParam, lParam);
}

